/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 */

#include "draw.h"

#define DRAW_PIXMAP(pixmap,x,y) \
gdk_draw_pixbuf(drawable, NULL, pixmap, 0,0,(x)*rozmer+1, \
(y)*rozmer+1,rozmer,rozmer,GDK_RGB_DITHER_NORMAL,0,0);

#define DRAW_PIXMAP_VYREZ(pixmap,px,py,pdx,pdy,x,y) \
gdk_draw_pixbuf(drawable, NULL, pixmap,(px),(py),(x), \
(y),(pdx),(pdy),GDK_RGB_DITHER_NORMAL,0,0)
				      
#define GET_ORIENT_BAB(level,x) ((x<(level)->player.column)?1:0)

static const gchar* TIPS_DESCRIPTION[]={
	N_("This is your figure. You may move or change orientation with arrow keys.You may shoot with Space key, and you may lay a mine with Ctrl key. After your round is done, you confirm it by pressing Ins key or numpad 0."),
	N_("This enemy can move 3 squares during each round. He is killed by 1 shot or by a mine."),
	N_("This enemy can move 4 squares during each round. He is killed by 1 shot or by a mine."),
	N_("This enemy can move 3 squares during each round. He is killed by 2 shots or by a mine."),
	N_("You may show your enemy's range. This color signifies the area where enemy may enter during 1 round. You can show 1 Balabak's range with right-click of mouse button, or all Balabaks' range by pressing r key, or persistently show range by menu: Tools/Options/Show range permanently."),
	N_("This feature is indestructible, unaffected by shots and explosions."),
	N_("One shot destroys a brick wall square, leaving a clear avenue."),
	N_("Water cannot be crossed by player or enemy, but you can shoot across it."),
	N_("A one-time explosive charge, detonated by the connected trigger. The range of destruction includes all adjacent squares, so 9 in all (3 x 3). Player shot won't cause detonation."),
	N_("This item is wired to, and activates, a nearby device (Dynamite, Dragon's head or Bomb catapult). It can be tripped by player or enemy."),
	N_("This item has a number in the left corner. The number counts down the rounds of play before detonation."),
	N_("Shoot to detonate. The range of destruction is 9 squares (3 x 3). If other Explosive barrels are in the range, they blow too."),
	N_("The dragon belches destructive flame, all along the line of its orientation, each time the connected trigger is stepped on. And Dragonfire sets off Explosive barrels."),
	N_("This hurls a bomb to the corresponding target mark, decimating a 3-by-3 square area, each time the connected trigger is stepped on."),
	N_("Target of bomb catapult (initially obscured if brick wall occupies same square)"),
	N_("When you step onto this square, you come out at the corresponding Teleport exit. Enemy can also make use. You could teleport a mine by depositing it to the entrance square."),
	N_("Note: Teleport passage is temporarily locked when the exit square is occupied by a figure."),
	N_("Step on this square to acquire 5 more mines. Note: an explosion would destroy the cache."),
	N_("Step on this square to acquire 5 more cannon shots. Note: an explosion would destroy the cache."),
	N_("Scoring is based on how few rounds you needed to completely dispose of the enemy, as compared to an assessed average for that particular level. A bonus is added based on the level's difficulty ranking."),
    N_("This enemy can move 3 squares per round and throws a fireball with each step"),
	N_("This enemy can move 5 squares per round and is killed by 3 shots or by a mine."),
    N_("Renews full count of moves/shots, for player or enemy; one time use only."),
    N_("Abruptly ends allotted moves/shots, for player or enemy; one time use only."),
	N_("Player can push stone to adjacent free square, which action counts as 2 moves.")
	/*N_("You are awarded some points for winning the level, ranging from 10 points at first, to 100 points for level #200; in simple direct proportion to the level number. And upon that, a point is added (or subtracted) for each round of play conserved (or in excess of) the norm assessed for completion of that particular level. If there is no variance, the score is termed \"Par\".")*/
};

static const gchar* TIPS_NAME[]={
	N_("Player"),
	N_("Blue Balabak"),
	N_("Green Balabak"),
	N_("Red Balabak"),
	N_("Range"),
	N_("Steel wall"),
	N_("Brick wall"),
	N_("Water"),
	N_("Dynamite"),
	N_("Trigger"),
	N_("Time bomb"),
	N_("Explosive barrel"),
	N_("Dragon's head"),
	N_("Bomb catapult"),
	N_("Target"),
	N_("Teleport - enter"),
	N_("Teleport - exit"),
	N_("Mines"),
	N_("Projectiles"),
	N_("Score"),
    N_("Black Balabak"),
	N_("Orange Balabak"),
	N_("Elixir"),
	N_("Arrest"),
	N_("Stone")
};


static GdkPixbuf *drak, *water;
static GdkPixbuf *player[4],*balaban[5][2],*barier,*retarder,*missiles;
static GdkPixbuf *mine,*mines,*fire[6],*fuel,*cable[6],*charge[2]; 
static GdkPixbuf *dragon[4],*push[4],*entry,*output,*sraf[4],*stop;
static GdkPixbuf *catapult[2][3],*target,*descriptions[3];
static GdkPixbuf *stimul, *topor, *orb;
static GdkColor *BLUE,*LGREY,*DGREY,*VDGREY;
static GTimeVal lastBadMove;
static PangoFontDescription *font;
static PangoContext* pc;
static guint rozmer=32;

gboolean porovnejCas(GTimeVal* cas)
{
    GTimeVal now; 
    g_get_current_time(&now);
    if(cas->tv_sec==now.tv_sec){
        if(cas->tv_usec+250000>now.tv_usec)return FALSE;
    }else{
        if(cas->tv_sec+1==now.tv_sec){
            if(cas->tv_usec-750000>now.tv_usec)return FALSE;
		}
    }
    return TRUE;  
}

void setRozmer(Level* level,guint x)
{
  if(rozmer==x) return;
  rozmer=x;
  unrefPixmaps();
  initDraw();
  loadPixmaps(level);
  initDrawFont(level);
}

static void paintCislo(guint32 x,guint32 y, guint cislo, GdkDrawable *drawable,Level* level)
{
    GdkGC* gc=gdk_gc_new(drawable);
    gchar chcislo[5];

    PangoLayout* pl=pango_layout_new(pc);

    pango_layout_set_font_description(pl,font);
    g_snprintf(chcislo,4,"%d",cislo);
    pango_layout_set_text(pl,chcislo,cislo>9?2:1);
    gdk_draw_layout(drawable,gc,x*rozmer+2,y*rozmer-1,pl);

    g_object_unref((gpointer)gc);
    g_object_unref((gpointer)pl);
}    

static void drawField(guint32 field,guint32 x,guint32 y,GdkDrawable *drawable,
    Level* level)
{
    switch(field){
        case 1:
            DRAW_PIXMAP(retarder,x,y);
            break;
        case 2:
            DRAW_PIXMAP(barier,x,y);
            break;
        case 3:
            DRAW_PIXMAP(missiles,x,y);
            break;
        case 4:
            break;
        case 5:
            DRAW_PIXMAP(mines,x,y);
            break;
        case 6:
            DRAW_PIXMAP(fuel,x,y);
            break;
        case 7:
        case 8:
        case 9:
        case 10:                        
            DRAW_PIXMAP(dragon[field-7],x,y);
            break;
        case 11:
        case 12:
        case 13:
        case 14:
        case 15:
        case 16:
            DRAW_PIXMAP(cable[field-11],x,y);
            break;
        case 17:
            DRAW_PIXMAP(entry,x,y);
            guint out=INDEX(level->map.parx[INDEX(x,y)],level->map.pary[INDEX(x,y)]);
            if(level->map.parx[out]>0){
	            GdkGC* gc= gdk_gc_new(drawable);
	            gdk_gc_set_foreground (gc, BLUE);
	            gdk_draw_rectangle(drawable,gc,TRUE,x*rozmer+2,y*rozmer+2,8,8);
   	            g_object_unref((gpointer)gc);
	        }
            if(IS_MINE(level->map.field[out]) || is_balaban(level,
                level->map.parx[INDEX(x,y)],level->map.pary[INDEX(x,y)]) ||
                ( level->map.parx[INDEX(x,y)]==level->player.column && 
                level->map.pary[INDEX(x,y)]==level->player.raw))
                DRAW_PIXMAP(stop,x,y);
	    if(level->map.parx[out]>0)
	        paintCislo(x,y,level->map.parx[out],drawable,level);
            break;
        case 18:
            DRAW_PIXMAP(output,x,y);
	        if(level->map.parx[INDEX(x,y)]>0){
	            GdkGC* gc= gdk_gc_new(drawable);
	            gdk_gc_set_foreground (gc, BLUE);
	            gdk_draw_rectangle(drawable,gc,TRUE,x*rozmer+2,y*rozmer+2,8,8);
   	            g_object_unref((gpointer)gc);
	            paintCislo(x,y,level->map.parx[INDEX(x,y)],drawable,level);
	        }    
            break;
        case 19:
            DRAW_PIXMAP(charge[0],x,y);
            break;
        case 20:
            DRAW_PIXMAP(charge[1],x,y);
            paintCislo(x,y,level->map.parx[INDEX(x,y)],drawable,level);
            break;
        case 21:
        case 22:
        case 23:
        case 24:
            DRAW_PIXMAP(push[field-21],x,y);
            break;
        case 25:
            DRAW_PIXMAP(mine,x,y);
            break;
        case 26:
            DRAW_PIXMAP(output,x,y);
            DRAW_PIXMAP(mine,x,y);
	    if(level->map.parx[INDEX(x,y)]>0)
	        paintCislo(x,y,level->map.parx[INDEX(x,y)],drawable,level);
            break;
        case 27:
        case 28:
        case 29:
        case 30:
        case 31:
        case 32:
            DRAW_PIXMAP(cable[field-27],x,y);
            DRAW_PIXMAP(mine,x,y);
            break;
        case 33:
        case 34:
            DRAW_PIXMAP(catapult[field-33][0],x,y);
            break;
        case 35:
            DRAW_PIXMAP(target,x,y);
	    break;
        case 36:
            DRAW_PIXMAP(target,x,y);
	    	DRAW_PIXMAP(mine,x,y);
	    break;
        case 37:
        case 38:
        case 39:
            DRAW_PIXMAP(descriptions[field-37],x,y);
            break;
        case 40:
        case 41:
        case 42:
            DRAW_PIXMAP(descriptions[field-40],x,y);
	    	DRAW_PIXMAP(mine,x,y);
            break;
        case 43:
	    	DRAW_PIXMAP(water,x,y);
	   		break;
        case 44:
	    	DRAW_PIXMAP(stimul,x,y);
	   		break;
        case 45:
	    	DRAW_PIXMAP(topor,x,y);
	   		break;
        case 46:
		case 47:
        case 48:
		case 49:
        case 50:
		case 51:
        case 52:
		case 53:
        case 54:
		case 55:
	    	DRAW_PIXMAP(orb,x,y);
	   		break;
    } 
}    

static void oznacStatus(Level* level,gint i)
{
    switch(i){
        case 1:
            gtk_entry_select_region(level->info->moves1,0,1);
            break;
        case 2:
            gtk_entry_select_region(level->info->moves2,0,1);
            break;
        case 3:
            gtk_entry_select_region(level->info->projectile1,0,3);
            break;
        case 4:
            gtk_entry_select_region(level->info->projectile2,0,3);
            break;
    }    
}

static GdkPixbuf *create_pixbuf(GString* prefix,const gchar* name)
{
    GError* error=NULL;
    GdkPixbuf *vysl;
    GString* str=g_string_new(name);

    g_string_prepend (str,prefix->str);
    vysl=gdk_pixbuf_new_from_file (str->str, &error);
    g_string_free(str,TRUE);

    return vysl;
}

static gboolean test(GString* prefix,const gchar* name)
{
    gboolean vysl;
    GString* str=g_string_new(name);

    g_string_prepend (str,prefix->str);
    vysl=g_file_test(str->str,G_FILE_TEST_EXISTS);
    g_string_free(str,TRUE);

    return vysl;
}

static void animateFirePole(gint orientation,guint32 x,guint32 y,gboolean animate,
                            GdkDrawable *drawable,gboolean zacatek)
{
    guint x1,x2,x3,y1,y2,y3,dx1,dx2,dx3,dy1,dy2,dy3,i;
    switch(orientation){
        case 0:
            x1=11;x2=9;x3=7;y1=8;y2=10;y3=13;dx1=10;dx2=14;dx3=18;dy1=2;dy2=3;dy3=5;
            break;
        case 1:
            x1=22;x2=19;x3=14;y1=11;y2=9;y3=7;dx1=2;dx2=3;dx3=5;dy1=10;dy2=14;dy3=18;
            break;
        case 2:
            x1=11;x2=9;x3=7;y1=22;y2=19;y3=14;dx1=10;dx2=14;dx3=18;dy1=2;dy2=3;dy3=5;
            break;
        case 3:
            x1=8;x2=10;x3=13;y1=11;y2=9;y3=7;dx1=2;dx2=3;dx3=5;dy1=10;dy2=14;dy3=18;
            break;
    }
    if(rozmer!=32){
		x1=x1*rozmer/32;x2=x2*rozmer/32;x3=x3*rozmer/32;
		y1=y1*rozmer/32;y2=y2*rozmer/32;y3=y3*rozmer/32;
        dx1=dx1*rozmer/32;dx2=dx2*rozmer/32;dx3=dx3*rozmer/32;
		dy1=dy1*rozmer/32;dy2=dy2*rozmer/32;dy3=dy3*rozmer/32;
    }
    for(i=0;i<8;i++){
		guint ii=i*4*rozmer/32;
        switch(orientation){
            case 0:
                DRAW_PIXMAP_VYREZ(fire[1],x3,y3,dx3,dy3,x3+x*rozmer+1,rozmer*(y+1)-ii);
                DRAW_PIXMAP_VYREZ(fire[1],x2,y2,dx2,dy2,x2+x*rozmer+1,rozmer*(y+1)-3-ii);
                DRAW_PIXMAP_VYREZ(fire[1],x1,y1,dx1,dy1,x1+x*rozmer+1,rozmer*(y+1)-5-ii);
                if(zacatek){
		    		if(i>2){
		        		guint dy=(i*4-6)/2;
						if(rozmer!=32)dy=dy*rozmer/32;
						DRAW_PIXMAP_VYREZ(fire[3],0,0,rozmer,dy,x*rozmer+1,rozmer*(y+1)+5-ii);
						DRAW_PIXMAP_VYREZ(fire[3],0,rozmer-1-dy,rozmer,dy,x*rozmer+1,rozmer*(y+1)+dy+5-ii);
		  			}
				}else{
                    guint dy=((i+1)%3)?13:22;
					if(rozmer!=32)dy=dy*rozmer/32;                    
                    DRAW_PIXMAP_VYREZ(fire[3],0,0,rozmer,dy,x*rozmer+1,rozmer*(y+1)+5-ii);
				}
                break;
            case 1:
                DRAW_PIXMAP_VYREZ(fire[1],x3,y3,dx3,dy3,x*rozmer-4+ii,y3+rozmer*y+1);
                DRAW_PIXMAP_VYREZ(fire[1],x2,y2,dx2,dy2,x*rozmer+1+ii,y2+rozmer*y+1);
                DRAW_PIXMAP_VYREZ(fire[1],x1,y1,dx1,dy1,x*rozmer+4+ii,y1+rozmer*y+1);
				if(zacatek){
		    		if(i>2){
		        		guint dx=(i*4-6)/2;
						if(rozmer!=32)dx=dx*rozmer/32;
						DRAW_PIXMAP_VYREZ(fire[2],rozmer-1-dx,0,dx,rozmer,x*rozmer-4-dx+ii,rozmer*y+1);
						DRAW_PIXMAP_VYREZ(fire[2],0,0,dx,rozmer,x*rozmer-4-2*dx+ii,rozmer*y+1);
		  			}
				}else{
                    guint dx=((i+1)%3)?13:22;
					if(rozmer!=32)dx=dx*rozmer/32;
                    DRAW_PIXMAP_VYREZ(fire[2],rozmer-1-dx,0,dx,rozmer,x*rozmer-4-dx+ii,rozmer*y+1);
				}
                break;
            case 2:
                DRAW_PIXMAP_VYREZ(fire[1],x3,y3,dx3,dy3,x3+x*rozmer+1,rozmer*y-4+ii);
                DRAW_PIXMAP_VYREZ(fire[1],x2,y2,dx2,dy2,x2+x*rozmer+1,rozmer*y+1+ii);
                DRAW_PIXMAP_VYREZ(fire[1],x1,y1,dx1,dy1,x1+x*rozmer+1,rozmer*y+4+ii);
				if(zacatek){
		    		if(i>2){
		        		guint dy=(i*4-6)/2;
						if(rozmer!=32)dy=dy*rozmer/32;
						DRAW_PIXMAP_VYREZ(fire[3],0,rozmer-1-dy,rozmer,dy,x*rozmer+1,rozmer*y-4-dy+ii);
						DRAW_PIXMAP_VYREZ(fire[3],0,0,rozmer,dy,x*rozmer+1,rozmer*y-4-2*dy+ii);
		  			}
				}else{
		            guint dy=((i+1)%3)?13:22; 
					if(rozmer!=32)dy=dy*rozmer/32;  
                    DRAW_PIXMAP_VYREZ(fire[3],0,rozmer-1-dy,rozmer,dy,x*rozmer+1,rozmer*y-4-dy+ii);
				}
                break;
            case 3:
                DRAW_PIXMAP_VYREZ(fire[1],x3,y3,dx3,dy3,(x+1)*rozmer-ii,y3+rozmer*y+1);
                DRAW_PIXMAP_VYREZ(fire[1],x2,y2,dx2,dy2,(x+1)*rozmer-3-ii,y2+rozmer*y+1);
                DRAW_PIXMAP_VYREZ(fire[1],x1,y1,dx1,dy1,(x+1)*rozmer-5-ii,y1+rozmer*y+1);
                if(zacatek){
		    		if(i>2){
		        		guint dx=(i*4-6)/2;
						if(rozmer!=32)dx=dx*rozmer/32;
						DRAW_PIXMAP_VYREZ(fire[2],0,0,dx,rozmer,(x+1)*rozmer+5-ii,rozmer*y+1);
						DRAW_PIXMAP_VYREZ(fire[2],rozmer-1-dx,0,dx,rozmer,(x+1)*rozmer+dx+5-ii,rozmer*y+1);
		  			}
				}else{
                    guint dx=((i+1)%3)?13:22;
					if(rozmer!=32)dx=dx*rozmer/32;
                    DRAW_PIXMAP_VYREZ(fire[2],0,0,dx,rozmer,(x+1)*rozmer+5-ii,rozmer*y+1);
				}
        }
        gdk_flush();
        if(animate)g_usleep(4000);
    }
}

static void dokresliSraf(Level* level,guint32 x,guint32 y,GdkDrawable *drawable)
{
    GList* list;
    list=g_list_find(level->oznacB,GINT_TO_POINTER(INDEX(x,y)));
    if(list) DRAW_PIXMAP(sraf[0],x,y);
    list=g_list_find(level->oznacP,GINT_TO_POINTER(INDEX(x,y)));
    if(list) DRAW_PIXMAP(sraf[1],x,y);                                                                
}    

static void opravPixbuf(GdkPixbuf** pb,gboolean is,GdkPixbuf* zdroj,
		   int x,int y,int dx,int dy)
{
	if(is){
		if(!(*pb))
			(*pb)=gdk_pixbuf_new_subpixbuf(zdroj,x,y,dx,dy);
	}else{
    	if(*pb){
      		gdk_pixbuf_unref(*pb);
      		*pb=NULL;
    	}
	}
}

static void drawFieldForeground(guint32 x,guint32 y,GdkDrawable *drawable)
{
    GdkGC* gc= gdk_gc_new(drawable);

    gdk_gc_set_foreground (gc, BLUE);
    gdk_draw_rectangle(drawable,gc,TRUE,x*rozmer+1,y*rozmer+1,rozmer,rozmer);
    gdk_gc_set_foreground (gc, DGREY);
    gdk_draw_line(drawable,gc,x*rozmer+rozmer,y*rozmer+1,x*rozmer+rozmer,y*rozmer+rozmer);
    gdk_gc_set_foreground (gc, LGREY);
    gdk_draw_line(drawable,gc,x*rozmer+1,y*rozmer+1,x*rozmer+1,y*rozmer+rozmer);
    gdk_gc_set_foreground (gc, DGREY);
    gdk_draw_line(drawable,gc,x*rozmer+1,y*rozmer+rozmer,x*rozmer+rozmer,y*rozmer+rozmer);
    gdk_gc_set_foreground (gc, LGREY);
    gdk_draw_line(drawable,gc,x*rozmer+1,y*rozmer+1,x*rozmer+rozmer,y*rozmer+1);
    g_object_unref((gpointer)gc);
}

GtkWidget* createTableTips(guint32 ktere)
{
	GtkWidget *table, *label;
	gint pocet,i;
	gboolean konec=FALSE;
	GdkPixbuf *pix=NULL;
	GdkPixbuf *pom=NULL;
	GdkPixbuf *podklad,*pruhledny;
	GString* dir=g_string_new("");
    
	pocet=0;
	for(i=0;i<25;i++){
		if(ktere & (guint32)(1<<i)) pocet++;
	}

    g_assert(pocet>0);
    
#ifdef HAVE_CONFIG_H
#ifdef RELATIVE_PATH
    g_string_append(dir,"..");
    g_string_append_c(dir,G_DIR_SEPARATOR);
    g_string_append(dir,"share");
    g_string_append_c(dir,G_DIR_SEPARATOR);
    g_string_append(dir,"babala");
    g_string_append_c(dir,G_DIR_SEPARATOR);
#else
    g_string_append(dir,PACKAGE_DATA_DIR);
    g_string_append_c(dir,G_DIR_SEPARATOR);
    g_string_append(dir,"babala");
    g_string_append_c(dir,G_DIR_SEPARATOR);
#endif    
#endif    
    g_string_append(dir,"pixmaps");
    g_string_append_c(dir,G_DIR_SEPARATOR);

    podklad = create_pixbuf(dir,"podklad.png");
    pruhledny= create_pixbuf(dir,"pruhledny.png");

	table = gtk_table_new (pocet,3,FALSE);

	pocet=0;
	i=0;
	while(TRUE){
		while(!(ktere & (guint32)(1<<i))){
			i++;
            if(i>24){
				konec=TRUE;
				break;
			}
		}
		if(konec)break;
		switch(i){
        case 19: 
		case 0:
			pix=gdk_pixbuf_new_subpixbuf(podklad,0,288,32,32);
			pom=gdk_pixbuf_new_subpixbuf(pruhledny,0,0,32,32);
			gdk_pixbuf_composite (pom,pix,0,0,32,32,0.0,0.0,1.0,1.0,GDK_INTERP_NEAREST,255);
            break;                                 
		case 1:
			pix=gdk_pixbuf_new_subpixbuf(podklad,32,288,32,32);
			pom=gdk_pixbuf_new_subpixbuf(pruhledny,0,32,32,32);
			gdk_pixbuf_composite (pom,pix,0,0,32,32,0.0,0.0,1.0,1.0,GDK_INTERP_NEAREST,255);	
            break;
		case 2:
			pix=gdk_pixbuf_new_subpixbuf(podklad,64,288,32,32);
			pom=gdk_pixbuf_new_subpixbuf(pruhledny,64,32,32,32);
			gdk_pixbuf_composite (pom,pix,0,0,32,32,0.0,0.0,1.0,1.0,GDK_INTERP_NEAREST,255);
            break;
		case 3:
			pix=gdk_pixbuf_new_subpixbuf(podklad,96,288,32,32);
			pom=gdk_pixbuf_new_subpixbuf(pruhledny,128,32,32,32);
			gdk_pixbuf_composite (pom,pix,0,0,32,32,0.0,0.0,1.0,1.0,GDK_INTERP_NEAREST,255);	
            break;
		case 4:
			pix=gdk_pixbuf_new_subpixbuf(podklad,128,288,32,32);
			pom=gdk_pixbuf_new_subpixbuf(pruhledny,32,64,32,32);
			gdk_pixbuf_composite (pom,pix,0,0,32,32,0.0,0.0,1.0,1.0,GDK_INTERP_NEAREST,255);	
            break;
		case 5:
			pix=gdk_pixbuf_new_subpixbuf(podklad,0,0,32,32);
			break;
		case 6:
			pix=gdk_pixbuf_new_subpixbuf(podklad,32,0,32,32);
			break;
		case 7:
			pix=gdk_pixbuf_new_subpixbuf(podklad,0,224,32,32);
			break;
		case 8:
			pix=gdk_pixbuf_new_subpixbuf(podklad,96,32,32,32);
			break;
		case 9:
			pix=gdk_pixbuf_new_subpixbuf(podklad,32,128,32,32);
			break;
		case 10:
			pix=gdk_pixbuf_new_subpixbuf(podklad,128,32,32,32);
			break;
		case 11:
			pix=gdk_pixbuf_new_subpixbuf(podklad,96,96,32,32);
			break;
		case 12:
			pix=gdk_pixbuf_new_subpixbuf(podklad,32,64,32,32);
			break;
		case 13:
			pix=gdk_pixbuf_new_subpixbuf(podklad,0,160,32,32);
			break;
		case 14:
			pix=gdk_pixbuf_new_subpixbuf(podklad,128,192,32,32);
			break;
		case 15:
			pix=gdk_pixbuf_new_subpixbuf(podklad,32,96,32,32);
			break;
		case 16:
			pix=gdk_pixbuf_new_subpixbuf(podklad,64,96,32,32);
			break;
		case 17:
			pix=gdk_pixbuf_new_subpixbuf(podklad,128,96,32,32);
			break;
		case 18:
			pix=gdk_pixbuf_new_subpixbuf(podklad,0,128,32,32);
			break;
        case 20:
			pix=gdk_pixbuf_new_subpixbuf(podklad,96,256,32,32);
			pom=gdk_pixbuf_new_subpixbuf(pruhledny,32,128,32,32);
			gdk_pixbuf_composite (pom,pix,0,0,32,32,0.0,0.0,1.0,1.0,GDK_INTERP_NEAREST,255);	
            break;
		case 21:
			pix=gdk_pixbuf_new_subpixbuf(podklad,128,256,32,32);
			pom=gdk_pixbuf_new_subpixbuf(pruhledny,96,128,32,32);
			gdk_pixbuf_composite (pom,pix,0,0,32,32,0.0,0.0,1.0,1.0,GDK_INTERP_NEAREST,255);
			break;
		case 22:
			pix=gdk_pixbuf_new_subpixbuf(podklad,32,224,32,32);
			break;
		case 23:
			pix=gdk_pixbuf_new_subpixbuf(podklad,64,224,32,32);
			break;
		case 24:
			pix=gdk_pixbuf_new_subpixbuf(podklad,96,224,32,32);
			break;
		}

		if(pix != NULL){
	        setCellPix(table,pix,0,1,pocet,pocet+1);
			gdk_pixbuf_unref(pix);
		}
		if(pom){
		    gdk_pixbuf_unref(pom);
		    pom=NULL;
		}    
        setCell(table,1,2,pocet,pocet+1,_(TIPS_NAME[i]),FALSE);
        label=setCell(table,2,3,pocet,pocet+1,_(TIPS_DESCRIPTION[i]),TRUE);
        gtk_label_set_line_wrap(GTK_LABEL(label),TRUE);
		pocet++;
		i++;
	}
    gdk_pixbuf_unref(pruhledny);
	gdk_pixbuf_unref(podklad);
	
	return table;
}

void loadPixmaps(Level* level)
{
    GdkPixbuf *podklad,*pruhledny;
    GString* dir=g_string_new("");
    gint i;
    gboolean is[41];
    GList* list=level->balabans;
    Balaban* bal;
    
    for(i=0;i<41;i++) is[i]=FALSE;

#ifdef HAVE_CONFIG_H
#ifdef RELATIVE_PATH
    g_string_append(dir,"..");
    g_string_append_c(dir,G_DIR_SEPARATOR);
    g_string_append(dir,"share");
    g_string_append_c(dir,G_DIR_SEPARATOR);
    g_string_append(dir,"babala");
    g_string_append_c(dir,G_DIR_SEPARATOR);
#else
    g_string_append(dir,PACKAGE_DATA_DIR);
    g_string_append_c(dir,G_DIR_SEPARATOR);
    g_string_append(dir,"babala");
    g_string_append_c(dir,G_DIR_SEPARATOR);
#endif    
#endif    
    g_string_append(dir,"pixmaps");
    g_string_append_c(dir,G_DIR_SEPARATOR);

    if(level->player.mines>0)is[25]=TRUE;

    while(list){
        bal=(Balaban*)list->data;
        list=list->next;
        if(bal->type==0)is[26]=TRUE;
		if(bal->type==1)is[27]=TRUE;
		if(bal->type==2){
	    	is[26]=TRUE;
	    	is[28]=TRUE;
		}
		if(bal->type==3)is[36]=TRUE;
		if(bal->type==4){
	    	is[26]=TRUE;
	    	is[28]=TRUE;
			is[37]=TRUE;
		}
    }

    for(i=0;i<level->map.columns*level->map.raws;i++){
        switch(level->map.field[i]){
        	case 1:
	    		is[1]=TRUE;
        	    if(level->map.parx[i]>0){
	    	    if(level->map.parx[i]<25){
	    	        is[level->map.parx[i]]=TRUE;
	    	    }else{
			    	if(level->map.parx[i]<36)
	    	            is[level->map.parx[i]-4]=TRUE;
			    	else
			        	is[level->map.parx[i]-5]=TRUE;
				}
	    	}
            break;
        	case 3:
	    		is[3]=TRUE;
        	    break;
        	case 5:
	    		is[5]=TRUE;
	    		is[25]=TRUE;
	    		is[0]=TRUE;
        	    break;
        	case 6:
	    		is[6]=TRUE;
	    		is[0]=TRUE;
        	    break;
        	case 7:
        	case 8:
        	case 9:
        	case 10:
	    		is[level->map.field[i]]=TRUE;
	    		is[4]=TRUE;                        
        	    break;
        	case 19:
			case 20:
	    		is[level->map.field[i]]=TRUE;
	    		is[0]=TRUE;                        
            	break;
        	case 25:
	    		is[0]=TRUE;
            	is[25]=TRUE;                        
	    		break;
        	case 26:
	    		is[18]=TRUE;
            	is[0]=TRUE;                        
            	is[25]=TRUE;                        
	    		break;
        	case 27:
        	case 28:
        	case 29:
        	case 30:
        	case 31:
        	case 32:
	    		is[0]=TRUE;
            	is[level->map.field[i]-16]=TRUE;                        
            	is[25]=TRUE;                        
	    		break;
        	case 33:
        	case 34:
	    		is[0]=TRUE;
            	is[level->map.field[i]-4]=TRUE;                        
            	break;
			case 35:
	    		is[31]=TRUE;
            	break;
			case 36:
	    		is[31]=TRUE;
            	is[4]=TRUE;                        
	    		is[25]=TRUE;                        
            	break;
			case 37:
			case 38:
			case 39:
	    		is[level->map.field[i]-5]=TRUE;
            	break;
			case 40:
			case 41:
			case 42:
	    		is[level->map.field[i]-8]=TRUE;
            	is[4]=TRUE;                        
	    		is[25]=TRUE;                        
            	break;
			case 43:
	    		is[35]=TRUE;
            	break;
			case 44:
	    		is[38]=TRUE;
            	break;
			case 45:
	    		is[39]=TRUE;
            	break;
			case 46:
	    		is[40]=TRUE;
            	break;
			default:
	  			is[level->map.field[i]]=TRUE;
		}
    }
    if(test(dir,(rozmer==32)?"podklad.png":"podklad20.png")){
        if(rozmer==32){
		    podklad = create_pixbuf(dir,"podklad.png");
		    pruhledny= create_pixbuf(dir,"pruhledny.png");
		}else{
			podklad = create_pixbuf(dir,"podklad20.png");
		    pruhledny= create_pixbuf(dir,"pruhledny20.png");
        }
        
        opravPixbuf(&balaban[0][0],is[26],pruhledny,4*rozmer,0,rozmer,rozmer);
	    opravPixbuf(&balaban[0][1],is[26],pruhledny,0,rozmer,rozmer,rozmer);
        opravPixbuf(&balaban[1][0],is[27],pruhledny,rozmer,rozmer,rozmer,rozmer);
        opravPixbuf(&balaban[2][0],is[28],pruhledny,3*rozmer,rozmer,rozmer,rozmer);
	    opravPixbuf(&balaban[2][1],is[28],pruhledny,4*rozmer,rozmer,rozmer,rozmer);
		opravPixbuf(&balaban[3][0],is[36],pruhledny,0,4*rozmer,rozmer,rozmer);
	    opravPixbuf(&balaban[3][1],is[36],pruhledny,1*rozmer,4*rozmer,rozmer,rozmer);
		opravPixbuf(&balaban[4][0],is[37],pruhledny,2*rozmer,4*rozmer,rozmer,rozmer);
	    opravPixbuf(&balaban[4][1],is[37],pruhledny,3*rozmer,4*rozmer,rozmer,rozmer);

	    opravPixbuf(&barier,is[2],podklad,0,0,rozmer,rozmer);
        opravPixbuf(&retarder,is[1],podklad,rozmer,0,rozmer,rozmer);
        opravPixbuf(&missiles,is[3],podklad,0,4*rozmer,rozmer,rozmer);
        opravPixbuf(&mine,is[25],pruhledny,0,2*rozmer,rozmer,rozmer); 
        opravPixbuf(&mines,is[5],podklad,4*rozmer,3*rozmer,rozmer,rozmer);
        for(i=3;i<5;i++)
            opravPixbuf(&charge[i-3],is[16+i],podklad,i*rozmer,rozmer,rozmer,rozmer);
        opravPixbuf(&fuel,is[6],podklad,3*rozmer,3*rozmer,rozmer,rozmer);
        for(i=2;i<5;i++)
            opravPixbuf(&cable[i-2],is[9+i],podklad,i*rozmer,0,rozmer,rozmer);
        for(i=0;i<3;i++)
            opravPixbuf(&cable[i+3],is[14+i],podklad,i*rozmer,rozmer,rozmer,rozmer);
        opravPixbuf(&fire[0],is[0],podklad,4*rozmer,2*rozmer,rozmer,rozmer);
		opravPixbuf(&fire[4],is[0] | is[36],pruhledny,3*rozmer,3*rozmer,rozmer,rozmer);
        opravPixbuf(&fire[1],is[4],podklad,0,3*rozmer,rozmer,rozmer);
		opravPixbuf(&fire[2],is[4],pruhledny,rozmer,3*rozmer,rozmer,rozmer);
		opravPixbuf(&fire[3],is[4],pruhledny,2*rozmer,3*rozmer,rozmer,rozmer);
        opravPixbuf(&fire[5],is[36],pruhledny,4*rozmer,4*rozmer,rozmer,rozmer);
        for(i=0;i<4;i++)
            opravPixbuf(&dragon[i],is[i+7],podklad,i*rozmer,2*rozmer,rozmer,rozmer);
        for(i=1;i<5;i++)
            opravPixbuf(&push[i-1],is[20+i],podklad,i*rozmer,4*rozmer,rozmer,rozmer);
        opravPixbuf(&entry,is[17],podklad,rozmer,3*rozmer,rozmer,rozmer);
        opravPixbuf(&stop,is[17],pruhledny,0,3*rozmer,rozmer,rozmer);
        opravPixbuf(&output,is[18],podklad,2*rozmer,3*rozmer,rozmer,rozmer);
        for(i=0;i<3;i++)
            opravPixbuf(&catapult[0][i],is[29],podklad,i*rozmer,5*rozmer,rozmer,rozmer);
	    opravPixbuf(&catapult[1][0],is[30],podklad,3*rozmer,5*rozmer,rozmer,rozmer);
	    opravPixbuf(&catapult[1][1],is[30],podklad,4*rozmer,5*rozmer,rozmer,rozmer);
	    opravPixbuf(&catapult[1][2],is[30],podklad,0,6*rozmer,rozmer,rozmer);
        for(i=1;i<4;i++)
	        opravPixbuf(&descriptions[i-1],is[31+i],podklad,i*rozmer,6*rozmer,rozmer,rozmer);
        opravPixbuf(&target,is[31],podklad,4*rozmer,6*rozmer,rozmer,rozmer);
        opravPixbuf(&water,is[35],podklad,0,7*rozmer,rozmer,rozmer);
		opravPixbuf(&stimul,is[38],podklad,1*rozmer,7*rozmer,rozmer,rozmer);
        opravPixbuf(&topor,is[39],podklad,2*rozmer,7*rozmer,rozmer,rozmer);
		opravPixbuf(&orb,is[40],podklad,3*rozmer,7*rozmer,rozmer,rozmer);

        gdk_pixbuf_unref(pruhledny);
		gdk_pixbuf_unref(podklad);
    }
    g_string_free(dir,TRUE); 
}

void loadButtonPixbuf( GdkPixbuf **pixb)
{
    GdkPixbuf *ico;
    gint i;
    GString* dir=g_string_new("");
    
    for(i=0;i<8;i++) pixb[i]=(GdkPixbuf*)NULL;

#ifdef HAVE_CONFIG_H
#ifdef RELATIVE_PATH
    g_string_append(dir,"..");
    g_string_append_c(dir,G_DIR_SEPARATOR);
    g_string_append(dir,"share");
    g_string_append_c(dir,G_DIR_SEPARATOR);
    g_string_append(dir,"babala");
    g_string_append_c(dir,G_DIR_SEPARATOR);
#else
    g_string_append(dir,PACKAGE_DATA_DIR);
    g_string_append_c(dir,G_DIR_SEPARATOR);
    g_string_append(dir,"babala");
    g_string_append_c(dir,G_DIR_SEPARATOR);
#endif    
#endif    
    g_string_append(dir,"pixmaps");
    g_string_append_c(dir,G_DIR_SEPARATOR);

    if(test(dir,"ico.png")){
        ico = create_pixbuf(dir,"ico.png");
        for(i=0;i<8;i++)
             pixb[i]=gdk_pixbuf_new_subpixbuf(ico,0+i*20,0,20,20);
        gdk_pixbuf_unref(ico);
    }
    g_string_free(dir,TRUE); 
}
 
void unrefPixmaps()
{
    pango_font_description_free (font);
    g_object_unref((gpointer)pc);

    gint i,j;
    for(i=0;i<4;i++)
	gdk_pixbuf_unref(player[i]);
    for(i=0;i<5;i++){
        for(j=0;j<2;j++){
	    if(balaban[i][j])
	        gdk_pixbuf_unref(balaban[i][j]);
	}
    }
    for(i=0;i<4;i++)
        gdk_pixbuf_unref(sraf[i]);
    if(barier)
	gdk_pixbuf_unref(barier);
    if(retarder)
        gdk_pixbuf_unref(retarder);
    if(missiles)
        gdk_pixbuf_unref(missiles);
    if(mine)
        gdk_pixbuf_unref(mine); 
    if(mines)
        gdk_pixbuf_unref(mines);
    for(i=0;i<2;i++){
      if(charge[i])
            gdk_pixbuf_unref(charge[i]);
    }
    if(fuel)
        gdk_pixbuf_unref(fuel);
    for(i=0;i<6;i++){
        if(cable[i])
            gdk_pixbuf_unref(cable[i]);
    }
    for(i=0;i<6;i++){
        if(fire[i])
	    	gdk_pixbuf_unref(fire[i]);
    }
    for(i=0;i<4;i++){
      if(dragon[i])
            gdk_pixbuf_unref(dragon[i]);
    }
    for(i=0;i<4;i++){
        if(push[i])
            gdk_pixbuf_unref(push[i]);
    }
    if(entry)
        gdk_pixbuf_unref(entry);
    if(stop)
        gdk_pixbuf_unref(stop);
    if(output)
        gdk_pixbuf_unref(output);
    for(i=0;i<3;i++){
        for(j=0;j<2;j++){
	    	if(catapult[j][i])
                gdk_pixbuf_unref(catapult[j][i]);
		}
    }
    for(i=0;i<3;i++){
        if(descriptions[i])
	    gdk_pixbuf_unref(descriptions[i]);
    }
    if(target)
        gdk_pixbuf_unref(target);
    if(water)
        gdk_pixbuf_unref(water);
	if(stimul)
        gdk_pixbuf_unref(stimul);
    if(topor)
        gdk_pixbuf_unref(topor);
    if(orb)
        gdk_pixbuf_unref(orb);
}

void initDrawFont(Level* level)
{
/*#ifdef WIN
    PangoFontMap* fm;
    
    fm=(PangoFontMap*)pango_win32_font_map_for_display ();     
    pc=(PangoContext*)pango_context_new ();
    pango_context_set_font_map(pc,fm);
#else*/
    pc=gtk_widget_create_pango_context(GTK_WIDGET(level->info->area));
/*#endif*/
    
    font=pango_font_description_copy(pango_context_get_font_description(pc));

    pango_font_description_set_size(font,PANGO_SCALE*10);                                         
    pango_font_description_set_weight(font, PANGO_WEIGHT_BOLD);
    pango_font_description_set_family(font,"Ariel,Sans");
   
/*#ifdef WIN 
    pango_win32_shutdown_display () ; 
#endif*/
}

GdkPixbuf *initDraw()
{
    GdkPixbuf *podklad,*pruhledny;
    GString* dir=g_string_new("");
    gint i,j;

#ifdef HAVE_CONFIG_H
#ifdef RELATIVE_PATH
    g_string_append(dir,"..");
    g_string_append_c(dir,G_DIR_SEPARATOR);
    g_string_append(dir,"share");
    g_string_append_c(dir,G_DIR_SEPARATOR);
    g_string_append(dir,"babala");
    g_string_append_c(dir,G_DIR_SEPARATOR);
#else
    g_string_append(dir,PACKAGE_DATA_DIR);
    g_string_append_c(dir,G_DIR_SEPARATOR);
    g_string_append(dir,"babala");
    g_string_append_c(dir,G_DIR_SEPARATOR);
#endif    
#endif    
    g_string_append(dir,"pixmaps");
    g_string_append_c(dir,G_DIR_SEPARATOR);

    g_get_current_time(&lastBadMove); 
    BLUE=(GdkColor*)g_malloc(sizeof(GdkColor));
    BLUE->red=225*256;BLUE->green=235*256;BLUE->blue = 249*256;
    gdk_color_alloc(gdk_colormap_get_system(),BLUE);
    LGREY=(GdkColor*)g_malloc(sizeof(GdkColor));
    LGREY->red=245*256;LGREY->green=255*256;LGREY->blue = 255*256;
    gdk_color_alloc(gdk_colormap_get_system(),LGREY);
    DGREY=(GdkColor*)g_malloc(sizeof(GdkColor));
    DGREY->red=162*256;DGREY->green=162*256;DGREY->blue = 178*256;
    gdk_color_alloc(gdk_colormap_get_system(),DGREY);
    VDGREY=(GdkColor*)g_malloc(sizeof(GdkColor));
    VDGREY->red=105*256;VDGREY->green=105*256;VDGREY->blue = 105*256;
    gdk_color_alloc(gdk_colormap_get_system(),VDGREY);

    for(i=0;i<5;i++){
        for(j=0;j<2;j++)
	    balaban[i][j]=NULL;
    }
    barier=NULL;
    retarder=NULL;
    missiles=NULL;
    mine=NULL;
    mines=NULL;
    for(i=0;i<2;i++)
        charge[i]=NULL;
    fuel=NULL;
    for(i=0;i<6;i++)
        cable[i]=NULL;
    for(i=0;i<6;i++)
        fire[i]=NULL;
    for(i=0;i<4;i++)
        dragon[i]=NULL;
    for(i=0;i<4;i++)
        push[i]=NULL;
    entry=NULL;
    stop=NULL;
    output=NULL;
    for(i=0;i<3;i++){
        for(j=0;j<2;j++)
	    	catapult[j][i]=NULL;
    }
    for(i=0;i<3;i++)
        descriptions[i]=NULL;
    target=NULL;
    water=NULL;
	stimul=NULL;
    topor=NULL;
    orb=NULL;

	if(test(dir,(rozmer==32)?"podklad.png":"podklad20.png")){
        if(rozmer==32){
		    podklad = create_pixbuf(dir,"podklad.png");
		    pruhledny= create_pixbuf(dir,"pruhledny.png");
		}else{
			podklad = create_pixbuf(dir,"podklad20.png");
		    pruhledny= create_pixbuf(dir,"pruhledny20.png");
        }
        drak=create_pixbuf(dir,"drak.png");

        for(i=0;i<4;i++)
	    player[i] = gdk_pixbuf_new_subpixbuf(pruhledny,0+i*rozmer,0,rozmer,rozmer);
        balaban[1][1] = gdk_pixbuf_new_subpixbuf(pruhledny,2*rozmer,rozmer,rozmer,rozmer);
        for(i=1;i<5;i++)
            sraf[i-1] = gdk_pixbuf_new_subpixbuf(pruhledny,i*rozmer,2*rozmer,rozmer,rozmer);
        gdk_pixbuf_unref(pruhledny);
		gdk_pixbuf_unref(podklad);
		g_string_free(dir,TRUE); 
		return balaban[1][1];
    }
    g_string_free(dir,TRUE);    
    return NULL;
}

void drawWholeMap(GtkWidget *widget,GdkEventExpose *event,gpointer data)
{
    GdkDrawable *drawable;
    Level* level=(Level*)data;
    gint i,j;
    gint zacX,zacY,pocetX,pocetY;
    gboolean isT=is_tah(level);
    GList* list;
    GdkGC* gc;

    drawable=widget->window;
    gc= gdk_gc_new(drawable);
    zacX=event->area.x/rozmer;
    zacY=event->area.y/rozmer;
    pocetX=1+(event->area.x+event->area.width)/rozmer-event->area.x/rozmer;
    if(pocetX+zacX>level->map.columns)pocetX=level->map.columns-zacX;
    pocetY=1+(event->area.y+event->area.height)/rozmer-event->area.y/rozmer;;
    if(pocetY+zacY>level->map.raws)pocetY=level->map.raws-zacY;
    
    gdk_gc_set_foreground (gc, BLUE);
    gdk_draw_rectangle(drawable,gc,TRUE,
                       zacX*rozmer,zacY*rozmer,pocetX*rozmer+1,pocetY*rozmer+1);
    gdk_gc_set_foreground (gc, DGREY);
    for(i=zacX;i<zacX+pocetX+1;i++){
        if(i>0)
	  gdk_draw_line(drawable,gc,i*rozmer,0,i*rozmer,
			level->map.raws*rozmer);
    }
    gdk_gc_set_foreground (gc, LGREY);
    for(i=zacX;i<zacX+pocetX+1;i++){
	if(i<level->map.columns)        
	  gdk_draw_line(drawable,gc,i*rozmer+1,0,i*rozmer+1,
			level->map.raws*rozmer);
    }
    gdk_gc_set_foreground (gc, DGREY);
    for(i=zacY;i<zacY+pocetY+1;i++){
	if(i>0)
	  gdk_draw_line(drawable,gc,0,i*rozmer,
			level->map.columns*rozmer,i*rozmer);
    }
    gdk_gc_set_foreground (gc, LGREY);
    for(i=zacY;i<zacY+pocetY+1;i++){
	if(i<level->map.raws)        
	  gdk_draw_line(drawable,gc,0,i*rozmer+1,
			level->map.columns*rozmer,i*rozmer+1);
    }
    for(i=0;i<pocetX;i++){
        for(j=0;j<pocetY;j++){
            drawField(level->map.field[INDEX(zacX+i,zacY+j)],zacX+i,zacY+j,
                      drawable,level);
            gboolean ozn;
            gint type=get_balaban_type(level,zacX+i,zacY+j,&ozn);
            g_assert(type<5);
            if(type>=0){
                DRAW_PIXMAP(balaban[type][GET_ORIENT_BAB(level,zacX+i)],
			            	zacX+i,zacY+j);
           	    if(ozn) gdk_draw_rectangle(drawable,GTK_WIDGET(level->info->area)->style->black_gc,
                       FALSE,(zacX+i)*rozmer+2,(zacY+j)*rozmer+2,rozmer-2,rozmer-2);
            }else{    
                if(level->player.raw==(zacY+j) && level->player.column==(zacX+i)){
                    DRAW_PIXMAP(player[level->player.orientation],zacX+i,zacY+j);
                }
            }
            if(!isT)DRAW_PIXMAP(sraf[2],zacX+i,zacY+j);
        }
    }                
    list=level->oznacB;
    while(list){
        gint field=GPOINTER_TO_INT(list->data);
        list=list->next;
        guint32 x=field%level->map.columns; 
        guint32 y=field/level->map.columns;
        DRAW_PIXMAP(sraf[0],x,y);
    }
    list=level->oznacP;
    while(list){
        gint field=GPOINTER_TO_INT(list->data);
        list=list->next;
        guint32 x=field%level->map.columns; 
        guint32 y=field/level->map.columns;
        DRAW_PIXMAP(sraf[1],x,y);
    }   
    if(event->area.x+event->area.width>level->map.columns*rozmer+1){
        gdk_gc_set_foreground (gc, LGREY); 
        gdk_draw_line(drawable,gc,level->map.columns*rozmer+1,0,
                      level->map.columns*rozmer+1,level->map.raws*rozmer+1);
        if(event->area.x+event->area.width>level->map.columns*rozmer+2){
            gdk_gc_set_foreground (gc, VDGREY); 
            gdk_draw_rectangle(drawable,gc,
                       TRUE,level->map.columns*rozmer+2,0,
                       event->area.x + event->area.width-level->map.columns*rozmer-1,
                       level->map.raws*rozmer+2);
        }    
    }        
    if(event->area.y+event->area.height>level->map.raws*rozmer+1){
        gdk_gc_set_foreground (gc, LGREY); 
        gdk_draw_line(drawable,gc,0,level->map.raws*rozmer+1,
                      level->map.columns*rozmer+1,level->map.raws*rozmer+1);
        if(event->area.y+event->area.height>level->map.raws*rozmer+2){
            gdk_gc_set_foreground (gc, VDGREY); 
            gdk_draw_rectangle(drawable,gc,
                       TRUE,0,level->map.raws*rozmer+2,
                       event->area.x+event->area.width,
                       event->area.y+event->area.height-level->map.raws*rozmer-2);
        }    
    }
    if(event->area.x+event->area.width>level->map.columns*rozmer+2 ||
        event->area.y+event->area.height>level->map.raws*rozmer+2){ 
        gint cx,cy;
        gdk_drawable_get_size(drawable,&cx,&cy);
        if(cx>level->map.columns*rozmer+202 || cy>level->map.raws*rozmer+352){
            DRAW_PIXMAP_VYREZ(drak,0,0,200,350,cx-200,cy-350);    
        }    
    }    
    g_object_unref((gpointer)gc);
}

void redrawAllEnter(Level* level)
{
    gint i,j;
    GdkDrawable *drawable;
    
    drawable=(GTK_WIDGET(level->info->area))->window;
    for(i=0;i<level->map.columns;i++){
        for(j=0;j<level->map.raws;j++){
            if(level->map.field[INDEX(i,j)]==17)
                drawField(level->map.field[INDEX(i,j)],i,j,drawable,level);
        }
    }        
}    

void redrawPlayer(Level* level,guint32 oldX,guint32 oldY)
{
    GdkDrawable *drawable;
    drawable=(GTK_WIDGET(level->info->area))->window;

    if(level->map.field[INDEX(oldX,oldY)]==18 ||
        level->map.field[INDEX(level->player.column,level->player.raw)]==18)
        redrawAllEnter(level);
    
    drawFieldForeground(oldX,oldY,drawable);
    drawField(level->map.field[INDEX(oldX,oldY)],oldX,oldY,drawable,level);
    dokresliSraf(level,oldX,oldY,drawable);
    DRAW_PIXMAP(player[level->player.orientation],level->player.column,
                level->player.raw);
    dokresliSraf(level,level->player.column,level->player.raw,drawable);
    gdk_flush();
}

void redrawBalaban(Level* level,Balaban* ibalaban,guint32 oldX,guint32 oldY)
{
    GdkDrawable *drawable;
    drawable=(GTK_WIDGET(level->info->area))->window;
    
    if(level->map.field[INDEX(oldX,oldY)]==18 ||
        level->map.field[INDEX(ibalaban->column,ibalaban->raw)]==18)
        redrawAllEnter(level);
 
    drawFieldForeground(oldX,oldY,drawable);   
    drawField(level->map.field[INDEX(oldX,oldY)],oldX,oldY,drawable,level);
    dokresliSraf(level,oldX,oldY,drawable);
    DRAW_PIXMAP(balaban[ibalaban->type][GET_ORIENT_BAB(level,ibalaban->column)]
		,ibalaban->column,ibalaban->raw);
    if(ibalaban->param[1]==1) gdk_draw_rectangle(drawable,GTK_WIDGET(level->info->area)->style->black_gc,
                       FALSE,(ibalaban->column)*rozmer+2,(ibalaban->raw)*rozmer+2,rozmer-2,rozmer-2);
    gdk_flush();
}

void drawPole(Level* level,guint32 x,guint32 y)
{
    GdkDrawable *drawable;
    drawable=(GTK_WIDGET(level->info->area))->window;
    gboolean ozn;
    
    if(level->map.field[INDEX(x,y)]==18) redrawAllEnter(level);
    drawFieldForeground(x,y,drawable);
    drawField(level->map.field[INDEX(x,y)],x,y,drawable,level);
    gint type=get_balaban_type(level,x,y,&ozn);
    g_assert(type<5);
    if(type>=0){
        DRAW_PIXMAP(balaban[type][GET_ORIENT_BAB(level,x)],x,y);
        if(ozn) gdk_draw_rectangle(drawable,GTK_WIDGET(level->info->area)->style->black_gc,
                       FALSE,x*rozmer+2,y*rozmer+2,rozmer-2,rozmer-2);
    }else{    
        if(level->player.raw==y && level->player.column==x){
            DRAW_PIXMAP(player[level->player.orientation],x,y);
        }
    }
    dokresliSraf(level,x,y,drawable);
    gdk_flush();
}

void drawMine(Level* level,guint32 x,guint32 y)
{
    GdkDrawable *drawable;
    drawable=(GTK_WIDGET(level->info->area))->window;
    
    if(level->map.field[INDEX(x,y)]==26) redrawAllEnter(level);
    DRAW_PIXMAP(mine,x,y);
    gdk_flush();
}

void animateShotsX(Level* level,guint32 x,guint32 odY,gint32 doY,gboolean zasah)
{
    GdkDrawable *drawable;
    drawable=(GTK_WIDGET(level->info->area))->window;
    guint32 sx=x*rozmer+rozmer/2-1;
    guint32 syo=((gint32)odY<doY)?(odY*rozmer+rozmer-2):(odY*rozmer+3);
    gint32 syd=((gint32)odY<doY)?(doY*rozmer+3):(doY*rozmer+rozmer-2);
    guint32 oldY=0;
    gint32 i,krok;
    GdkPixbuf* old=NULL;
    GdkColormap *cmap=gdk_window_get_colormap(drawable);
    
    if(syd<0)syd=0;    
    if((gint32)odY<doY) krok=4;
    else krok=-4;
    for(i=syo;((gint32)odY<doY)?(i<syd):(i>syd);i+=krok){
        if(i!=syo)
            gdk_draw_pixbuf(drawable, NULL, old,0,0,sx,oldY,4,6,
				      GDK_RGB_DITHER_NORMAL,0,0);
		if(old)gdk_pixbuf_unref(old);
        old=gdk_pixbuf_get_from_drawable (NULL,drawable,cmap,sx,i,0,0,4,6);
        oldY=i;
        gdk_draw_rectangle(drawable,
            GTK_WIDGET(level->info->area)->style->black_gc,TRUE,sx,i,4,6);    
	gdk_flush();
        if(level->info->animate)g_usleep(10000);
    }
    if(i!=syo)
            gdk_draw_pixbuf(drawable, NULL,old,0,0,sx,oldY,4,6,
				      GDK_RGB_DITHER_NORMAL,0,0);
    if(zasah){
        drawFieldForeground(x,doY,drawable); 
        drawField(level->map.field[INDEX(x,doY)],x,doY,drawable,level);
    }
    if(old)gdk_pixbuf_unref(old);        
}
    
void animateShotsY(Level* level,guint32 y,guint32 odX,gint32 doX,gboolean zasah)
{
    GdkDrawable *drawable;
    drawable=(GTK_WIDGET(level->info->area))->window;
    guint32 sy=y*rozmer+rozmer/2-1;
    guint32 sxo=((gint32)odX<doX)?(odX*rozmer+rozmer-2):(odX*rozmer+3);
    gint32 sxd=((gint32)odX<doX)?(doX*rozmer+3):(doX*rozmer+rozmer-2);
    guint32 oldX=0;
    gint32 i,krok;
    GdkPixbuf* old=NULL;
    GdkColormap *cmap=gdk_window_get_colormap(drawable);

    if(sxd<0)sxd=0;
    if((gint32)odX<doX) krok=4;
    else krok=-4;
    for(i=sxo;((gint32)odX<doX)?(i<sxd):(i>sxd);i+=krok){
        if(i!=sxo)
            gdk_draw_pixbuf(drawable, NULL,old,0,0,oldX,sy,6,4,
				      GDK_RGB_DITHER_NORMAL,0,0);
		if(old)gdk_pixbuf_unref(old);
        old=gdk_pixbuf_get_from_drawable (NULL,drawable,cmap,i,sy,0,0,6,4);
        oldX=i;
        gdk_draw_rectangle(drawable,
            GTK_WIDGET(level->info->area)->style->black_gc,TRUE,i,sy,6,4);
	gdk_flush();    
        if(level->info->animate)g_usleep(10000);
    }
    if(i!=sxo)
            gdk_draw_pixbuf(drawable, NULL,old,0,0,oldX,sy,6,4,
				      GDK_RGB_DITHER_NORMAL,0,0);
    if(zasah){
        drawFieldForeground(doX,y,drawable); 
        drawField(level->map.field[INDEX(doX,y)],doX,y,drawable,level);
    }
    if(old)gdk_pixbuf_unref(old);
}

void animateCat(Level* level,guint32 odX,guint32 doX,gint32 y,GList* list)
{
    GdkDrawable *drawable;
    drawable=(GTK_WIDGET(level->info->area))->window;
    guint32 sy=y*rozmer;
    guint32 sxo=odX*rozmer+rozmer/2;
    gint32 sxd=(doX-odX)*rozmer;
    gint32 oldX=0;
    gint32 oldY=0;
    gint32 dx,dy;
    gint32 i,krok,max;
    GdkPixbuf* old=NULL;
    GdkColormap *cmap=gdk_window_get_colormap(drawable);

    max=(sxd>0)?sxd:-sxd;
    max /= 8;    
    if((gint32)odX<doX) krok=8;
    else krok=-8;

    if(level->info->animate)g_usleep(5000);
    DRAW_PIXMAP(catapult[level->map.field[INDEX(odX,y)]-33][1],odX,y);
    if(level->info->animate)g_usleep(20000);
    DRAW_PIXMAP(catapult[level->map.field[INDEX(odX,y)]-33][2],odX,y);

    for(i=0;i<=max;i++){
        if(i!=0)
            gdk_draw_pixbuf(drawable, NULL,old,0,0,oldX,(oldY>0)?oldY:0,rozmer/2,rozmer/2,
						GDK_RGB_DITHER_NORMAL,0,0);
		dx=sxo+i*krok;
        dy=sy+ i*i*krok*krok*7/max/80 - (i*((krok>0)?krok:-krok)*7/10);
		if(old)gdk_pixbuf_unref(old);  
        old=gdk_pixbuf_get_from_drawable (NULL,drawable,cmap,dx,(dy>0)?dy:0,0,0,rozmer/2,rozmer/2);
       	oldY=dy;
		oldX=dx;
        gdk_draw_arc(drawable,GTK_WIDGET(level->info->area)->style->black_gc,
           	TRUE,dx,dy,14*rozmer/32,14*rozmer/32,0,(360*64));
		gdk_flush();
        if(level->info->animate)g_usleep(20000);
    }
    gdk_draw_pixbuf(drawable, NULL,old,0,0,oldX,(oldY>0)?oldY:0,rozmer/2,rozmer/2,
			      GDK_RGB_DITHER_NORMAL,0,0);
    if(old)gdk_pixbuf_unref(old);        
    DRAW_PIXMAP(catapult[level->map.field[INDEX(odX,y)]-33][0],odX,y);
    animateVybuch(level,list);
}


void animateVybuch(Level* level,GList* list)
{
    GList* l2;
    GdkDrawable *drawable;
    gint isEnter=FALSE;
    
    drawable=(GTK_WIDGET(level->info->area))->window;
    l2=list;
    while(l2){
        gint field=GPOINTER_TO_INT(l2->data);
        l2=l2->next;
        guint32 x=field%level->map.columns; 
        guint32 y=field/level->map.columns;
        DRAW_PIXMAP(fire[0],x,y);
    }
    gdk_flush();
    if(level->info->animate)g_usleep(200000);
    l2=list;
    while(l2){
        gint field=GPOINTER_TO_INT(l2->data);
        l2=l2->next;
        guint32 x=field%level->map.columns; 
        guint32 y=field/level->map.columns;

	drawFieldForeground(x,y,drawable);
        drawField(level->map.field[field],x,y,drawable,level);
	DRAW_PIXMAP(fire[4],x,y);
    }    
    gdk_flush();
    if(level->info->animate)g_usleep(100000);
    l2=list;
    while(l2){
        gint field=GPOINTER_TO_INT(l2->data);
        l2=l2->next;
        guint32 x=field%level->map.columns; 
        guint32 y=field/level->map.columns;

        if(level->map.field[INDEX(x,y)]==18) isEnter=TRUE;
	    drawFieldForeground(x,y,drawable);
        drawField(level->map.field[field],x,y,drawable,level);
        dokresliSraf(level,x,y,drawable);
    }
    if(isEnter)redrawAllEnter(level);    
    gdk_flush();
}

void animateDraka(Level* level,guint32 x,guint32 y,GList* list)
{
    gint orientation;
    guint32 newX,newY;
    guint32 newField;
    GdkDrawable *drawable;
    gboolean zacatek=TRUE;
    gboolean isEnter=FALSE;
    
    drawable=(GTK_WIDGET(level->info->area))->window;
    newX=x;
    newY=y;
    orientation=(gint)level->map.field[INDEX(x,y)]-7;
    g_assert(orientation<4 && orientation>=0);
    while(TRUE){
        switch(orientation){
        case 0:
            newY--;
            break;
        case 1:
            newX++;
            break;
        case 2:
            newY++;
            break;
        case 3:
            newX--;
            break;
        }
        newField=INDEX(newX,newY); 
        if(newX<0 || newY<0 || newX>=level->map.columns || 
            newY>=level->map.raws || IS_RESIST(level->map.field[newField])){
            animateVybuch(level,list);
            break;
        }    
        animateFirePole(orientation,newX,newY,level->info->animate,
			drawable,zacatek);
	if(zacatek)zacatek=FALSE;
    }
    newX=x;
    newY=y;
    zacatek=TRUE;
    
    while(TRUE){
        newField=INDEX(newX,newY); 
        if(newX<0 || newY<0 || newX>=level->map.columns ||
            newY>=level->map.raws || IS_RESIST(level->map.field[newField])){ 
            if(zacatek) zacatek=FALSE;    
            else break;
        }
        if(level->map.field[newField]==18) isEnter=TRUE;
	    drawFieldForeground(newX,newY,drawable);
        drawField(level->map.field[newField],newX,newY,drawable,level);
        dokresliSraf(level,newX,newY,drawable);
        switch(orientation){
        case 0:
            newY--;
            break;
        case 1:
            newX++;
            break;
        case 2:
            newY++;
            break;
        case 3:
            newX--;
            break;
        }
    }
    if(isEnter)redrawAllEnter(level); 
    gdk_flush();
}

void drawBadMove(Level* level,gint co)
{
    GdkDrawable *drawable;
    drawable=(GTK_WIDGET(level->info->area))->window;
    GdkPixbuf* old;
    GdkColormap *cmap=gdk_window_get_colormap(drawable);

    if(!porovnejCas(&lastBadMove)) return;

    gtk_grab_add (GTK_WIDGET(level->info->area));    
    old=gdk_pixbuf_get_from_drawable (NULL,drawable,cmap,
        level->player.column*rozmer+1,level->player.raw*rozmer+1,0,0,rozmer,rozmer);
    
    DRAW_PIXMAP(sraf[3],level->player.column,level->player.raw);
    gdk_flush();
    g_usleep(150000); 
    DRAW_PIXMAP(old,level->player.column,level->player.raw);                 
    gdk_flush();
    g_usleep(70000);
    DRAW_PIXMAP(sraf[0],level->player.column,level->player.raw);
    gdk_flush();
    g_usleep(150000);
    DRAW_PIXMAP(old,level->player.column,level->player.raw);                 
    gdk_flush();
    g_usleep(70000);
    DRAW_PIXMAP(sraf[3],level->player.column,level->player.raw);
    gdk_flush();
    g_usleep(150000);                  
    DRAW_PIXMAP(old,level->player.column,level->player.raw);                 
    gdk_pixbuf_unref(old);
    gtk_grab_remove (GTK_WIDGET(level->info->area));
    oznacStatus(level,co);
    g_get_current_time(&lastBadMove);      
}

void animateHod(Level* level,guint32 odX,guint32 odY,guint32 doX,guint32 doY)
{
    guint32 i,faze;
    GdkDrawable *drawable;
    gboolean isEnter=FALSE;
    gboolean isFirst=TRUE;
    GdkPixbuf* old=NULL;    
    g_assert(odX==doX || doY==odY);
    drawable=(GTK_WIDGET(level->info->area))->window;
    GdkColormap *cmap=gdk_window_get_colormap(drawable);

	if(doY==odY){
		if(doX<odX){
			if(level->map.field[INDEX(odX,odY)]==18) isEnter=TRUE;
			for(i=odX;i>doX;i--){
				if(isFirst){
					isFirst=FALSE;
					faze=4;
				}else{
					faze=0;
				}
                if(level->map.field[INDEX(i-1,doY)]==18) isEnter=TRUE;
				if(old)gdk_pixbuf_unref(old);
				old=gdk_pixbuf_get_from_drawable (NULL,drawable,cmap,
        			(i-1)*rozmer,doY*rozmer,0,0,2*rozmer,rozmer);
				while (faze<8){
					DRAW_PIXMAP_VYREZ(fire[5],0,0,rozmer,rozmer,i*rozmer-rozmer*faze/8,doY*rozmer);
					gdk_flush();
					if(level->info->animate)g_usleep(12000);
    				gdk_draw_pixbuf(drawable, NULL,old,0,0,(i-1)*rozmer,doY*rozmer,rozmer*2,rozmer,
			      		GDK_RGB_DITHER_NORMAL,0,0);
					faze++;
					if(level->info->animate)faze+=2;
				}
			}
		}else{
			if(level->map.field[INDEX(odX,odY)]==18) isEnter=TRUE;
			for(i=odX;i<doX;i++){
				if(isFirst){
					isFirst=FALSE;
					faze=4;
				}else{
					faze=0;
				}
                if(level->map.field[INDEX(i+1,doY)]==18) isEnter=TRUE;
				if(old)gdk_pixbuf_unref(old);
				old=gdk_pixbuf_get_from_drawable (NULL,drawable,cmap,
        			i*rozmer,doY*rozmer,0,0,2*rozmer,rozmer);
				while (faze<8){
					DRAW_PIXMAP_VYREZ(fire[5],0,0,rozmer,rozmer,i*rozmer+rozmer*faze/8,doY*rozmer);
					gdk_flush();
					if(level->info->animate)g_usleep(12000);
					gdk_draw_pixbuf(drawable, NULL,old,0,0,i*rozmer,doY*rozmer,rozmer*2,rozmer,
			      		GDK_RGB_DITHER_NORMAL,0,0);
					faze++;
					if(level->info->animate)faze+=2;
				}
			}
		}
	}else{
		if(doY<odY){
			if(level->map.field[INDEX(odX,odY)]==18) isEnter=TRUE;
			for(i=odY;i>doY;i--){
				if(isFirst){
					isFirst=FALSE;
					faze=4;
				}else{
					faze=0;
				}
                if(level->map.field[INDEX(doX,i-1)]==18) isEnter=TRUE;
				if(old)gdk_pixbuf_unref(old);
				old=gdk_pixbuf_get_from_drawable (NULL,drawable,cmap,
        			doX*rozmer,(i-1)*rozmer,0,0,rozmer,2*rozmer);
				while (faze<8){
					DRAW_PIXMAP_VYREZ(fire[5],0,0,rozmer,rozmer,doX*rozmer,i*rozmer-rozmer*faze/8);
    				gdk_flush();
					if(level->info->animate)g_usleep(12000);
					gdk_draw_pixbuf(drawable, NULL,old,0,0,doX*rozmer,(i-1)*rozmer,rozmer,2*rozmer,
			      		GDK_RGB_DITHER_NORMAL,0,0);
					faze++;
                    if(level->info->animate)faze+=2;
				}
			}
		}else{
			if(level->map.field[INDEX(odX,odY)]==18) isEnter=TRUE;
			for(i=odY;i<doY;i++){
				if(isFirst){
					isFirst=FALSE;
					faze=4;
				}else{
					faze=0;
				}
                if(level->map.field[INDEX(doX,i+1)]==18) isEnter=TRUE;
				if(old)gdk_pixbuf_unref(old);
				old=gdk_pixbuf_get_from_drawable (NULL,drawable,cmap,
        			doX*rozmer,i*rozmer,0,0,rozmer,2*rozmer);
				while (faze<8){
					DRAW_PIXMAP_VYREZ(fire[5],0,0,rozmer,rozmer,doX*rozmer,i*rozmer+rozmer*faze/8);
					gdk_flush();
					if(level->info->animate)g_usleep(12000);
					gdk_draw_pixbuf(drawable, NULL,old,0,0,doX*rozmer,i*rozmer,rozmer,2*rozmer,
			      		GDK_RGB_DITHER_NORMAL,0,0);
					faze++;
					if(level->info->animate)faze+=2;
				}
			}
		}
	}

	DRAW_PIXMAP(fire[4],doX,doY);
	gdk_flush();
	if(level->info->animate)g_usleep(250000);
    drawFieldForeground(doX,doY,drawable);
	gdk_flush();
	g_usleep(20000);
	drawField(level->map.field[INDEX(doX,doY)],doX,doY,drawable,level);
	dokresliSraf(level,doX,doY,drawable);

    if(isEnter)redrawAllEnter(level); 
	if(old)gdk_pixbuf_unref(old);
    gdk_flush();
}
                   
